@@ -1,5 +1,4 @@
-#!/usr/bin/env python
-# -*- mode:python; tab-width: 2; coding: utf-8 -*-
+# -*- mode:python; coding: utf-8 -*-
 
 """
 Logging module for sleipnir
@@ -14,11 +13,10 @@ To use, simply 'import log' and use 'log instance
 from __future__ import absolute_import
 
 __author__  = "Carlos Martin <cmartin@liberalia.net>"
-__status__  = "alpha"
-__version__ = "0.1"
-__date__    = "19 January 2010"
 __license__ = "See LICENSE file for details"
 
+import os
+import re
 import sys
 import logging
 import logging.handlers
@@ -38,14 +36,14 @@ class LoggerColorFormatter(logging.Formatter):
             severity
             """
 
-            codes = {\
+            codes = {
                 None:       (0,   0),
                 'DEBUG':    (0,   2),  # grey
                 'INFO':     (0,   0),  # normal
                 'WARNING':  (1,  34),  # blue
                 'ERROR':    (1,  31),  # red
                 'CRITICAL': (1, 101),  # black, red
-                }
+            }
             return (chr(27) + '[%d;%dm') % codes[level]
 
         retval = logging.Formatter.format(self, record)
@@ -61,17 +59,17 @@ class LoggerHandler(object):
     severity levels for each of the handlers used by sleipnir
     """
 
+    SYSLOG_DEV = '/dev/log'
+
     __params = {
         'syslog': {
-            'hlevels':
-                logging.WARNING,
-            },
+            'hlevels': logging.WARNING,
+        },
         'stderr': {
-            'hlevels':
-                logging.DEBUG,
-            },
+            'hlevels': logging.DEBUG,
+        },
         None: {
-            }
+        }
     }
     handlers = {}
 
@@ -85,8 +83,15 @@ class LoggerHandler(object):
 
         if handler in ('file',) and 'logfile' in kwargs:
             hdlr = logging.FileHandler(kwargs['logfile'])
+
+        # This special socket doesn't exists on Maemo5 SDK.
+        # Check it existence first
         elif handler in ('syslog', 'unix',):
-            hdlr = logging.handlers.SysLogHandler('/dev/log')
+            if os.path.exists(self.SYSLOG_DEV):
+                hdlr = logging.handlers.SysLogHandler(self.SYSLOG_DEV)
+            else:
+                hdlr = logging.StreamHandler(sys.stderr)
+
         elif handler in ('stderr',):
             hdlr = logging.StreamHandler(sys.stderr)
         elif handler in (None,):
@@ -142,57 +147,99 @@ class Logger(logging.Logger):
         self.setLevel(kwargs['level'])
 
 
-class _LoggerDict(dict):
-    """A Custom dict to allow access to key elements like
-    attributes"""
+class _LoggerDict(object):
+    """
+    A Custom dict to allow access to key elements like
+    attributes
+    """
+
+    def __init__(self, dct):
+        # compute an arg string
+        self._dct = dict(dct)
+
+        # override levels if a --log=LOGNAME:LEVEL,... is passed at
+        # command line
+        levels = "".join(sys.argv)
+        if 'log' in levels:
+            self.parse_levels(levels)
 
     def __getattr__(self, name):
-        if name in self:
-            return self[name]
+        if name in self._dct:
+            return self._dct[name]
         raise AttributeError(name)
 
+    def parse_levels(self, levels):
+        """Upgrade logging levels with ones provided with levels arg"""
+        regex = re.compile("--foo=(.*)[ |\t]?")
+        for level in regex.match(levels).groups()[0].split(','):
+            name, _, level = levels.partition(':')
+            try:
+                # If only devel is provided, partition will put it
+                # under name
+                level = level or name
+                level = getattr(logging, level.upper())
+            except KeyError:
+                continue
+            # set log for one case only or globally
+            # pylint: disable-msg=W0106
+            self._dct[name].setLevel(level) \
+                if name in self._dct else   \
+                [logg.setLevel(level) for logg in self._dct.itervalues()]
+
 
 # pylint: disable-msg=C0103
-log = _LoggerDict({\
+log = _LoggerDict(
+    {
         'components':
-            Logger("COMPONENTS",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("COMPONENTS",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'console':
-            Logger("CONSOLE",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("CONSOLE",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'controller':
-            Logger("CONTROLLER",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("CONTROLLER",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'core':
-            Logger("CORE",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("CORE",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'fremantle':
-            Logger("FREMANTLE",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("FREMANTLE",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'parser':
-            Logger("PARSER",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("PARSER",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'plugins':
-            Logger("PLUGINS",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("PLUGINS",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'profiles':
-            Logger("PROFILES",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("PROFILES",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'sections':
-            Logger("TSP-SECTIONS",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
+        Logger("TSP-SECTIONS",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
         'tsplib':
-            Logger("TSPLIB",
-                   handlers=('syslog', 'stderr',),
-                   level=logging.WARNING),
-
-})
+        Logger("TSPLIB",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
+        'shell':
+        Logger("SHELL",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
+        'handset':
+        Logger("HANDSET",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
+        'views':
+        Logger("VIEWS",
+               handlers=('syslog', 'stderr',),
+               level=logging.WARNING),
+    })
